/*
 * RenderPipelineTest.java
 */

package de.ba_ravensburg.gdv.controller;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

import junit.framework.TestCase;
import de.ba_ravensburg.gdv.model.Scene;
import de.ba_ravensburg.gdv.model.Triangle;
import de.ba_ravensburg.gdv.view.View3DPanel;

/**
 * @author Ansgar Wiechers
 */
public class RenderPipelineTest extends TestCase {

  private final int HORIZONTAL = Triangle.HORIZONTAL;
  private final int VERTICAL = Triangle.VERTICAL;
  private final int X = Scene.X;
  private final int Y = Scene.Y;
  private final int Z = Scene.Z;
  
  /**
   * Constructor for RenderPipelineTest.
   * @param arg0
   */
  public RenderPipelineTest(String arg0) {
    super(arg0);
  }
  
  public void testPutTriangles() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    Triangle triangle1 = new Triangle(-1.0, 1.0, 0.0,
                                      0.0, 1.0, 1.0,
                                      1.0, 1.0, -1.0);
    Triangle triangle2 = new Triangle(1.0, -1.0, 2.0,
                                      -2.0, -1.0, 0.0,
                                      0.0, -1.0, -1.0);
    Triangle triangle3 = new Triangle(1.0, 1.0, -1.0,
                                      2.0, 0.0, 3.5,
                                      5.0, 1.0, 2.0);
    Triangle triangle4 = new Triangle(-2.0, 2.0, 0.0,
                                      -4.0, 3.0, -2.0,
                                      -2.0, 1.0, 1.0);
    Triangle triangle5 = new Triangle(-1.0, -1.0, 2.0,
                                      1.0, 1.0, 3.0,
                                      0.0, 1.0, 2.0);
    Triangle triangle6 = new Triangle(-1.0, 1.0, -2.0,
                                      1.0, 1.0, -2.0,
                                      0.0, 1.0, -3.0);
    Triangle triangle7 = new Triangle(-1.0, 1.0, 0.0,
                                      1.0, 1.0, -1.0,
                                      0.0, -2.0, 1.0);
    double aperture = 30.0;
    
    triangles.add(triangle1);
    triangles.add(triangle2);
    triangles.add(triangle3);
    triangles.add(triangle4);
    triangles.add(triangle5);
    triangles.add(triangle6);
    triangles.add(triangle7);
    rp.put(triangles, aperture);
    
    Class c = rp.getClass();
    
    try {
      Field queue = c.getDeclaredField("queue"); //$NON-NLS-1$
      queue.setAccessible(true);
      
      assertTrue(((LinkedList)queue.get(rp)).contains(triangle1));
      assertFalse(((LinkedList)queue.get(rp)).contains(triangle2));
      assertFalse(((LinkedList)queue.get(rp)).contains(triangle3));
      assertFalse(((LinkedList)queue.get(rp)).contains(triangle4));
      assertFalse(((LinkedList)queue.get(rp)).contains(triangle5));
      assertFalse(((LinkedList)queue.get(rp)).contains(triangle6));
      assertFalse(((LinkedList)queue.get(rp)).contains(triangle7));
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testPutTrianglesEmpty() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    double aperture = 30.0;
    
    rp.put(triangles, aperture);
    
    Class c = rp.getClass();
    
    try {
      Field queue = c.getDeclaredField("queue"); //$NON-NLS-1$
      queue.setAccessible(true);
      
      assertTrue(((LinkedList)queue.get(rp)).isEmpty());
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testPutTrianglesInvalidList() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = null;
    double aperture = 30.0;
    
    try {
      rp.put(triangles, aperture);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testPutTrianglesInvalidApertureLow() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 0.0, 1.0);
    double aperture = 0.0;
    
    triangles.add(triangle);
    
    try {
      rp.put(triangles, aperture);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testPutTrianglesInvalidApertureHigh() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 0.0, 1.0);
    double aperture = 180.1;
    
    triangles.add(triangle);
    
    try {
      rp.put(triangles, aperture);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testGet() {
    RenderPipeline rp = new RenderPipeline();
    double aperture = 45.0;
    double[] p11 = {3.0, 2.0, 0.0};
    double[] p12 = {0.0, 3.0, 0.0};
    double[] p13 = {0.0, 2.0, 1.0};
    double[] p21 = {-2.0, 4.0, 0.0};
    double[] p22 = {1.0, 4.0, 2.0};
    double[] p23 = {2.0, 4.0, -1.0};
    int[] p11_2D = new int[2];
    int[] p12_2D = new int[2];
    int[] p13_2D = new int[2];
    int[] p21_2D = new int[2];
    int[] p22_2D = new int[2];
    int[] p23_2D = new int[2];
    Triangle triangle1 = new Triangle(p11[X], p11[Y], p11[Z],
                                      p12[X], p12[Y], p12[Z],
                                      p13[X], p13[Y], p13[Z]);
    Triangle triangle2 = new Triangle(p21[X], p21[Y], p21[Z],
                                      p22[X], p22[Y], p22[Z],
                                      p23[X], p23[Y], p23[Z]);
    Triangle2D triangle;
    HashSet<Triangle> triangles3D = new HashSet<Triangle>();
    LinkedList triangles2D;
    double screenDistance = 2.0;
    double pixelPerUnit = (double)View3DPanel.SIZE / (2 * screenDistance * Math.tan(Math.toRadians(aperture)));
    
    triangles3D.add(triangle1);
    triangles3D.add(triangle2);
    
    rp.put(triangles3D, aperture);
    
    p11_2D[0] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) + p11[X] / p11[Y]));
    p11_2D[1] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) - p11[Z] / p11[Y]));
    p12_2D[0] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) + p12[X] / p12[Y]));
    p12_2D[1] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) - p12[Z] / p12[Y]));
    p13_2D[0] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) + p13[X] / p13[Y]));
    p13_2D[1] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) - p13[Z] / p13[Y]));
    
    p21_2D[0] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) + p21[X] / p21[Y]));
    p21_2D[1] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) - p21[Z] / p21[Y]));
    p22_2D[0] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) + p22[X] / p22[Y]));
    p22_2D[1] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) - p22[Z] / p22[Y]));
    p23_2D[0] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) + p23[X] / p23[Y]));
    p23_2D[1] = (int)Math.round(pixelPerUnit * screenDistance * (Math.tan(Math.toRadians(aperture)) - p23[Z] / p23[Y]));
    
    triangles2D = rp.get();
    
    triangle = (Triangle2D)triangles2D.removeFirst();
    // One pixel deviation is acceptable (due to rounding errors).
    assertTrue(Math.abs(triangle.xpoints[0] - p21_2D[0]) <= 1);
    assertTrue(Math.abs(triangle.ypoints[0] - p21_2D[1]) <= 1);
    assertTrue(Math.abs(triangle.xpoints[1] - p22_2D[0]) <= 1);
    assertTrue(Math.abs(triangle.ypoints[1] - p22_2D[1]) <= 1);
    assertTrue(Math.abs(triangle.xpoints[2] - p23_2D[0]) <= 1);
    assertTrue(Math.abs(triangle.ypoints[2] - p23_2D[1]) <= 1);
    assertEquals(triangle2.getColor(), triangle.getColor());
    
    triangle = (Triangle2D)triangles2D.removeFirst();
    // One pixel deviation is acceptable (due to rounding errors).
    assertTrue(Math.abs(triangle.xpoints[0] - p11_2D[0]) <= 1);
    assertTrue(Math.abs(triangle.ypoints[0] - p11_2D[1]) <= 1);
    assertTrue(Math.abs(triangle.xpoints[1] - p12_2D[0]) <= 1);
    assertTrue(Math.abs(triangle.ypoints[1] - p12_2D[1]) <= 1);
    assertTrue(Math.abs(triangle.xpoints[2] - p13_2D[0]) <= 1);
    assertTrue(Math.abs(triangle.ypoints[2] - p13_2D[1]) <= 1);
    assertEquals(triangle1.getColor(), triangle.getColor());
  }
  
  public void testGetFromEmpty() {
    RenderPipeline rp = new RenderPipeline();
    
    assertTrue(rp.get().isEmpty());
  }
  
  public void testPutTriangle() {
    RenderPipeline rp = new RenderPipeline();
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 0.0, 1.0);
    
    Class c = rp.getClass();
    Class[] params = {Triangle.class};
    Object[] args = {triangle};
    
    try {
      Method put = c.getDeclaredMethod("put", params); //$NON-NLS-1$
      put.setAccessible(true);
      Field queue = c.getDeclaredField("queue"); //$NON-NLS-1$
      queue.setAccessible(true);
      
      assertFalse(((LinkedList)queue.get(rp)).contains(triangle));
      
      put.invoke(rp, args);
      
      assertTrue(((LinkedList)queue.get(rp)).contains(triangle));
    } catch (Exception e) {
      fail("Failed: field or method inaccessible."); //$NON-NLS-1$
    }
  }

  public void testIsBehindCamera() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    Triangle triangle1 = new Triangle(-1.0, 1.0, 0.0,
                                      0.0, 1.0, 1.0,
                                      1.0, 1.0, -1.0);
    Triangle triangle2 = new Triangle(1.0, -1.0, 2.0,
                                      -2.0, -1.0, 0.0,
                                      0.0, -1.0, -1.0);
    Triangle triangle3 = new Triangle(1.0, 1.0, -1.0,
                                      2.0, 0.0, 3.5,
                                      5.0, 1.0, 2.0);
    Triangle triangle4 = new Triangle(-2.0, 2.0, 0.0,
                                      -4.0, 3.0, -2.0,
                                      -2.0, 1.0, 1.0);
    Triangle triangle5 = new Triangle(-1.0, -1.0, 2.0,
                                      1.0, 1.0, 3.0,
                                      0.0, 1.0, 2.0);
    Triangle triangle6 = new Triangle(-1.0, 1.0, -2.0,
                                      1.0, 1.0, -2.0,
                                      0.0, 1.0, -3.0);
    Triangle triangle7 = new Triangle(-1.0, 1.0, 0.0,
                                      1.0, 1.0, -1.0,
                                      0.0, -2.0, 1.0);
    Boolean result;
    Iterator it;
    
    triangles.add(triangle1);
    triangles.add(triangle2);
    triangles.add(triangle3);
    triangles.add(triangle4);
    triangles.add(triangle5);
    triangles.add(triangle6);
    triangles.add(triangle7);
    
    Class c = rp.getClass();
    Class[] params = {Triangle.class};
    
    try {
      Method method = c.getDeclaredMethod("isBehindCamera", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      it = triangles.iterator();
      while (it.hasNext()) {
        Object[] args = {(Triangle)it.next()};
        result = (Boolean)method.invoke(rp, args);
        if (args[0] == triangle2)
          assertTrue(result.booleanValue());
        else
          assertFalse(result.booleanValue());
      }
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }

  public void testIsLeftOfCamera() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    Triangle triangle1 = new Triangle(-1.0, 1.0, 0.0,
                                      0.0, 1.0, 1.0,
                                      1.0, 1.0, -1.0);
    Triangle triangle2 = new Triangle(1.0, -1.0, 2.0,
                                      -2.0, -1.0, 0.0,
                                      0.0, -1.0, -1.0);
    Triangle triangle3 = new Triangle(1.0, 1.0, -1.0,
                                      2.0, 0.0, 3.5,
                                      5.0, 1.0, 2.0);
    Triangle triangle4 = new Triangle(-2.0, 2.0, 0.0,
                                      -4.0, 3.0, -2.0,
                                      -2.0, 1.0, 1.0);
    Triangle triangle5 = new Triangle(-1.0, -1.0, 2.0,
                                      1.0, 1.0, 3.0,
                                      0.0, 1.0, 2.0);
    Triangle triangle6 = new Triangle(-1.0, 1.0, -2.0,
                                      1.0, 1.0, -2.0,
                                      0.0, 1.0, -3.0);
    Triangle triangle7 = new Triangle(-1.0, 1.0, 0.0,
                                      1.0, 1.0, -1.0,
                                      0.0, -2.0, 1.0);
    Triangle thisTriangle;
    Boolean result;
    Iterator it;
    
    triangles.add(triangle1);
    triangles.add(triangle2);
    triangles.add(triangle3);
    triangles.add(triangle4);
    triangles.add(triangle5);
    triangles.add(triangle6);
    triangles.add(triangle7);
    
    Class c = rp.getClass();
    Class[] params = {double.class, double.class, double.class};
    
    try {
      Method method = c.getDeclaredMethod("isLeftOfCamera", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      it = triangles.iterator();
      while (it.hasNext()) {
        thisTriangle = (Triangle)it.next();
        Object[] args = {new Double(thisTriangle.getAngleOfA(HORIZONTAL)),
                         new Double(thisTriangle.getAngleOfB(HORIZONTAL)),
                         new Double(thisTriangle.getAngleOfC(HORIZONTAL))};
        result = (Boolean)method.invoke(rp, args);
        if (thisTriangle == triangle3)
          assertTrue(result.booleanValue());
        else
          assertFalse(result.booleanValue());
      }
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testIsRightOfCamera() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    Triangle triangle1 = new Triangle(-1.0, 1.0, 0.0,
                                      0.0, 1.0, 1.0,
                                      1.0, 1.0, -1.0);
    Triangle triangle2 = new Triangle(1.0, -1.0, 2.0,
                                      -2.0, -1.0, 0.0,
                                      0.0, -1.0, -1.0);
    Triangle triangle3 = new Triangle(1.0, 1.0, -1.0,
                                      2.0, 0.0, 3.5,
                                      5.0, 1.0, 2.0);
    Triangle triangle4 = new Triangle(-2.0, 2.0, 0.0,
                                      -4.0, 3.0, -2.0,
                                      -2.0, 1.0, 1.0);
    Triangle triangle5 = new Triangle(-1.0, -1.0, 2.0,
                                      1.0, 1.0, 3.0,
                                      0.0, 1.0, 2.0);
    Triangle triangle6 = new Triangle(-1.0, 1.0, -2.0,
                                      1.0, 1.0, -2.0,
                                      0.0, 1.0, -3.0);
    Triangle triangle7 = new Triangle(-1.0, 1.0, 0.0,
                                      1.0, 1.0, -1.0,
                                      0.0, -2.0, 1.0);
    Triangle thisTriangle;
    Boolean result;
    Iterator it;
    
    triangles.add(triangle1);
    triangles.add(triangle2);
    triangles.add(triangle3);
    triangles.add(triangle4);
    triangles.add(triangle5);
    triangles.add(triangle6);
    triangles.add(triangle7);
    
    Class c = rp.getClass();
    Class[] params = {double.class, double.class, double.class};
    
    try {
      Method method = c.getDeclaredMethod("isRightOfCamera", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      it = triangles.iterator();
      while (it.hasNext()) {
        thisTriangle = (Triangle)it.next();
        Object[] args = {new Double(thisTriangle.getAngleOfA(HORIZONTAL)),
                         new Double(thisTriangle.getAngleOfB(HORIZONTAL)),
                         new Double(thisTriangle.getAngleOfC(HORIZONTAL))};
        result = (Boolean)method.invoke(rp, args);
        if (thisTriangle == triangle4)
          assertTrue(result.booleanValue());
        else
          assertFalse(result.booleanValue());
      }
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }

  public void testIsAboveCamera() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    Triangle triangle1 = new Triangle(-1.0, 1.0, 0.0,
                                      0.0, 1.0, 1.0,
                                      1.0, 1.0, -1.0);
    Triangle triangle2 = new Triangle(1.0, -1.0, 2.0,
                                      -2.0, -1.0, 0.0,
                                      0.0, -1.0, -1.0);
    Triangle triangle3 = new Triangle(1.0, 1.0, -1.0,
                                      2.0, 0.0, 3.5,
                                      5.0, 1.0, 2.0);
    Triangle triangle4 = new Triangle(-2.0, 2.0, 0.0,
                                      -4.0, 3.0, -2.0,
                                      -2.0, 1.0, 1.0);
    Triangle triangle5 = new Triangle(-1.0, -1.0, 2.0,
                                      1.0, 1.0, 3.0,
                                      0.0, 1.0, 2.0);
    Triangle triangle6 = new Triangle(-1.0, 1.0, -2.0,
                                      1.0, 1.0, -2.0,
                                      0.0, 1.0, -3.0);
    Triangle triangle7 = new Triangle(-1.0, 1.0, 0.0,
                                      1.0, 1.0, -1.0,
                                      0.0, -2.0, 1.0);
    Triangle thisTriangle;
    Boolean result;
    Iterator it;
    
    triangles.add(triangle1);
    triangles.add(triangle2);
    triangles.add(triangle3);
    triangles.add(triangle4);
    triangles.add(triangle5);
    triangles.add(triangle6);
    triangles.add(triangle7);
    
    Class c = rp.getClass();
    Class[] params = {double.class, double.class, double.class};
    
    try {
      Method method = c.getDeclaredMethod("isAboveCamera", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      it = triangles.iterator();
      while (it.hasNext()) {
        thisTriangle = (Triangle)it.next();
        Object[] args = {new Double(thisTriangle.getAngleOfA(VERTICAL)),
                         new Double(thisTriangle.getAngleOfB(VERTICAL)),
                         new Double(thisTriangle.getAngleOfC(VERTICAL))};
        result = (Boolean)method.invoke(rp, args);
        if (thisTriangle == triangle5)
          assertTrue(result.booleanValue());
        else
          assertFalse(result.booleanValue());
      }
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }

  public void testIsBelowCamera() {
    RenderPipeline rp = new RenderPipeline();
    HashSet<Triangle> triangles = new HashSet<Triangle>();
    Triangle triangle1 = new Triangle(-1.0, 1.0, 0.0,
                                      0.0, 1.0, 1.0,
                                      1.0, 1.0, -1.0);
    Triangle triangle2 = new Triangle(1.0, -1.0, 2.0,
                                      -2.0, -1.0, 0.0,
                                      0.0, -1.0, -1.0);
    Triangle triangle3 = new Triangle(1.0, 1.0, -1.0,
                                      2.0, 0.0, 3.5,
                                      5.0, 1.0, 2.0);
    Triangle triangle4 = new Triangle(-2.0, 2.0, 0.0,
                                      -4.0, 3.0, -2.0,
                                      -2.0, 1.0, 1.0);
    Triangle triangle5 = new Triangle(-1.0, -1.0, 2.0,
                                      1.0, 1.0, 3.0,
                                      0.0, 1.0, 2.0);
    Triangle triangle6 = new Triangle(-1.0, 1.0, -2.0,
                                      1.0, 1.0, -2.0,
                                      0.0, 1.0, -3.0);
    Triangle triangle7 = new Triangle(-1.0, 1.0, 0.0,
                                      1.0, 1.0, -1.0,
                                      0.0, -2.0, 1.0);
    Triangle thisTriangle;
    Boolean result;
    Iterator it;
    
    triangles.add(triangle1);
    triangles.add(triangle2);
    triangles.add(triangle3);
    triangles.add(triangle4);
    triangles.add(triangle5);
    triangles.add(triangle6);
    triangles.add(triangle7);
    
    Class c = rp.getClass();
    Class[] params = {double.class, double.class, double.class};
    
    try {
      Method method = c.getDeclaredMethod("isBelowCamera", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      it = triangles.iterator();
      while (it.hasNext()) {
        thisTriangle = (Triangle)it.next();
        Object[] args = {new Double(thisTriangle.getAngleOfA(VERTICAL)),
                         new Double(thisTriangle.getAngleOfB(VERTICAL)),
                         new Double(thisTriangle.getAngleOfC(VERTICAL))};
        result = (Boolean)method.invoke(rp, args);
        if (thisTriangle == triangle6)
          assertTrue(result.booleanValue());
        else
          assertFalse(result.booleanValue());
      }
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }

  public void testIsTurnedAway() {
    RenderPipeline rp = new RenderPipeline();
    Triangle triangle1 = new Triangle(1.0, 1.0, 0.0,
                                      0.0, 1.0, 1.0,
                                      0.0, 1.0, 0.0);
    Triangle triangle2 = new Triangle(1.0, 1.0, 0.0,
                                      0.0, 1.0, 0.0,
                                      0.0, 1.0, 1.0);
    Triangle triangle3 = new Triangle(1.0, 1.0, 0.0,
                                      2.0, 2.0, 0.0,
                                      1.0, 1.0, 1.0);
    Triangle triangle4 = new Triangle(1.0, 0.0, 0.0,
                                      2.0, 1.0, 0.0,
                                      1.0, 0.0, 1.0);
    Triangle triangle5 = new Triangle(1.0, 2.0, 0.0,
                                      2.0, 3.0, 0.0,
                                      1.0, 2.0, 1.0);
    Boolean result;
    
    Class c = rp.getClass();
    Class[] params = {Triangle.class};
    Object[] args1 = {triangle1};
    Object[] args2 = {triangle2};
    Object[] args3 = {triangle3};
    Object[] args4 = {triangle4};
    Object[] args5 = {triangle5};
    
    try {
      Method method = c.getDeclaredMethod("isTurnedAway", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      result = (Boolean)method.invoke(rp, args1);
      assertTrue(result.booleanValue());
      
      result = (Boolean)method.invoke(rp, args2);
      assertFalse(result.booleanValue());
      
      result = (Boolean)method.invoke(rp, args3);
      assertTrue(result.booleanValue());
      
      result = (Boolean)method.invoke(rp, args4);
      assertFalse(result.booleanValue());
      
      result = (Boolean)method.invoke(rp, args5);
      assertTrue(result.booleanValue());
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testCoordinate2D() {
    RenderPipeline rp = new RenderPipeline();
    double aperture = 30.0;
    double value1 = 10.0;
    double distance1 = 0.1;
    double value2 = 10.0;
    double distance2 = 1.0;
    double value3 = (double)Integer.MAX_VALUE + 1.0;
    double distance3 = 1.0;
    double value4 = (double)Integer.MIN_VALUE - 1.0;
    double distance4 = 1.0;
    int expectedResult1, expectedResult2, expectedResult3, expectedResult4;
    Integer result;
    
    Class c = rp.getClass();
    Class[] params = {double.class, double.class};
    Object[] args1 = {new Double(value1), new Double(distance1)};
    Object[] args2 = {new Double(value2), new Double(distance2)};
    Object[] args3 = {new Double(value3), new Double(distance3)};
    Object[] args4 = {new Double(value4), new Double(distance4)};
    
    rp.put(new HashSet(), aperture);
    
    expectedResult1 = (int)Math.round((1.0 + value1 / distance1 / Math.tan(Math.toRadians(aperture))) * View3DPanel.SIZE / 2.0);
    expectedResult2 = (int)Math.round((1.0 + value2 / distance2 / Math.tan(Math.toRadians(aperture))) * View3DPanel.SIZE / 2.0);
    expectedResult3 = Integer.MAX_VALUE;
    expectedResult4 = Integer.MIN_VALUE;
    
    try {
      Method method = c.getDeclaredMethod("coordinate2D", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      result = (Integer)method.invoke(rp, args1);
      assertEquals(expectedResult1, result.intValue());
      
      result = (Integer)method.invoke(rp, args2);
      assertEquals(expectedResult2, result.intValue());
      
      result = (Integer)method.invoke(rp, args3);
      assertEquals(expectedResult3, result.intValue());
      
      result = (Integer)method.invoke(rp, args4);
      assertEquals(expectedResult4, result.intValue());
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testCoordinate2DInvalid() {
    RenderPipeline rp = new RenderPipeline();
    double aperture = 30.0;
    double value = 10.0;
    double distance = 0.0;
    
    Class c = rp.getClass();
    Class[] params = {double.class, double.class};
    Object[] args = {new Double(value), new Double(distance)};
    
    rp.put(new HashSet(), aperture);
    
    try {
      Method method = c.getDeclaredMethod("coordinate2D", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      method.invoke(rp, args);
    } catch (InvocationTargetException ite) {
      if (ite.getTargetException().toString().endsWith("IllegalArgumentException")) //$NON-NLS-1$
        return; 
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }

}
