/*
 * Triangle2D.java
 */

package de.ba_ravensburg.gdv.controller;

import java.awt.Color;

/**
 * We need polygons with three points, that have a color, so we specialize
 * <i>Polygon</i> with these features.
 * 
 * @see java.awt.Polygon
 * 
 * @author Ansgar Wiechers
 */
public class Triangle2D extends java.awt.Polygon {

  private Color color;
  
  /**
   * Each 2D triangle must have three points and a color. To ensure this, the
   * default constructor is hidden.
   */
  private Triangle2D() {}
  
  /**
   * Construct a new 2D triangle with given coordinates and given color.
   * 
   * @see java.awt.Polygon#Polygon(int[], int[], int)
   * 
   * @param xpoints  an array of <i>x</i> coordinates.
   * @param ypoints  an array of <i>y</i> coordinates.
   * @param npoints  the total number of points in the triangle.
   * @param theColor the color of the triangle.
   * @throws IllegalArgumentException if total number of points != 3.
   */
  Triangle2D(int[] xpoints, int[] ypoints, int npoints, Color theColor)
    throws IllegalArgumentException {
    super(xpoints, ypoints, npoints);
    
    if (xpoints.length < 3 || ypoints.length < 3)
      throw new IllegalArgumentException();
    
    color = theColor;
  }
  
  /**
   * Get the color of this 2D triangle.
   * 
   * @return the color of the 2D triangle.
   */
  public Color getColor() {
    return color;
  }

}
