/*
 * Triangle2DTest.java
 */

package de.ba_ravensburg.gdv.controller;

import java.awt.Color;
import java.lang.reflect.Field;

import junit.framework.TestCase;

/**
 * @author Ansgar Wiechers
 */
public class Triangle2DTest extends TestCase {

  /**
   * Constructor for Triangle2DTest.
   * @param arg0
   */
  public Triangle2DTest(String arg0) {
    super(arg0);
  }

  public void testTriangle2D() {
    Color theColor = Color.CYAN;
    int[] xpoints = {1, 2, 3};
    int[] ypoints = {4, 5, 6};
    int npoints = 3;
    Triangle2D triangle = new Triangle2D(xpoints, ypoints, npoints, theColor);
    
    Class c = triangle.getClass();
    
    try {
      Field color = c.getDeclaredField("color"); //$NON-NLS-1$
      color.setAccessible(true);
      
      assertEquals(theColor, (Color)color.get(triangle));
      
      assertEquals(npoints, triangle.npoints);
      assertEquals(npoints, triangle.xpoints.length);
      assertEquals(npoints, triangle.ypoints.length);
      
      assertEquals(xpoints[0], triangle.xpoints[0]);
      assertEquals(xpoints[1], triangle.xpoints[1]);
      assertEquals(xpoints[2], triangle.xpoints[2]);
      
      assertEquals(ypoints[0], triangle.ypoints[0]);
      assertEquals(ypoints[1], triangle.ypoints[1]);
      assertEquals(ypoints[2], triangle.ypoints[2]);
    } catch (Exception e) {
      fail("Failed: field inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testTriangle2DIncompatibleArgs() {
    try {
      Triangle2D triangle = new Triangle2D(new int[] {1, 2},
                                           new int[] {1, 2, 3},
                                           3, Color.BLACK);
    } catch (IndexOutOfBoundsException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testTriangle2DInvalidLowXPoints() {
    try {
      Triangle2D triangle = new Triangle2D(new int[] {1, 2},
                                           new int[] {1, 2, 3},
                                           2, Color.BLACK);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testTriangle2DInvalidLowYPoints() {
    try {
      Triangle2D triangle = new Triangle2D(new int[] {1, 2, 3},
                                           new int[] {1, 2},
                                           2, Color.BLACK);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testGetColor() {
    Color color = Color.GRAY;
    Triangle2D triangle = new Triangle2D(new int[] {1, 2, 3},
                                         new int[] {1, 2, 3},
                                         3, color);
    
    assertEquals(color, triangle.getColor());
  }

}
