/*
 * TriangleTest.java
 */

package de.ba_ravensburg.gdv.model;

import java.awt.Color;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import junit.framework.TestCase;

/**
 * @author Ansgar Wiechers
 */
public class TriangleTest extends TestCase {

  private final int X = Scene.X;
  private final int Y = Scene.Y;
  private final int Z = Scene.Z;
  private final int LEFTRIGHT = Scene.LEFTRIGHT;
  private final int UPDOWN = Scene.UPDOWN;
  private final int HORIZONTAL = Triangle.HORIZONTAL;
  private final int VERTICAL = Triangle.VERTICAL;
  private final double PRECISION = 0.0001;

  /**
   * Constructor for TriangleTest.
   * @param arg0
   */
  public TriangleTest(String arg0) {
    super(arg0);
  }

  /*
   * Test for constructor Triangle(double[], double[], double[])
   */
  public void testTriangledoubleArraydoubleArraydoubleArray() {
    double[] p1 = {1.0, 1.0, 3.0};
    double[] p2 = {1.0, 3.0, 1.0};
    double[] p3 = {3.0, 1.0, 1.0};
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    
    // Check pointA
    assertEquals(p1[X], triangle.getPointA()[X], PRECISION);
    assertEquals(p1[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(p1[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(p2[X], triangle.getPointB()[X], PRECISION);
    assertEquals(p2[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(p2[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(p3[X], triangle.getPointC()[X], PRECISION);
    assertEquals(p3[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(p3[Z], triangle.getPointC()[Z], PRECISION);
    // Check color
    assertEquals(Triangle.DEFAULT_COLOR, triangle.getColor());
  }

  /*
   * Test for constructor Triangle(double[], double[], double[], Color)
   */
  public void testTriangledoubleArraydoubleArraydoubleArrayColor() {
    double[] p1 = {1.0, 1.0, 3.0};
    double[] p2 = {1.0, 3.0, 1.0};
    double[] p3 = {3.0, 1.0, 1.0};
    Color color = new Color(255, 0, 0);
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z],
                                     color);
    
    // Check pointA
    assertEquals(p1[X], triangle.getPointA()[X], PRECISION);
    assertEquals(p1[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(p1[Z], triangle.getPointA()[Z], PRECISION);
    // Check pointB
    assertEquals(p2[X], triangle.getPointB()[X], PRECISION);
    assertEquals(p2[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(p2[Z], triangle.getPointB()[Z], PRECISION);
    // Check pointC
    assertEquals(p3[X], triangle.getPointC()[X], PRECISION);
    assertEquals(p3[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(p3[Z], triangle.getPointC()[Z], PRECISION);
    // Check color
    assertEquals(color, triangle.getColor());
  }
  
  /*
   * Test for constructor Triangle(double[], double[], double[]) with invalid
   * Argument. Two points are equal.
   */
  public void testTriangleInvalidTwoPoints() {
    try {
      Triangle triangle = new Triangle(1.0, 1.0, 3.0,
                                       1.0, 1.0, 3.0,
                                       3.0, 1.0, 1.0);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  /*
   * Test for constructor Triangle(double[], double[], double[]) with invalid
   * Argument. All three points are equal.
   */
  public void testTriangleInvalidThreePoints() {
    try {
      Triangle triangle = new Triangle(0.0, 0.0, 0.0,
                                       0.0, 0.0, 0.0,
                                       0.0, 0.0, 0.0);
    } catch (IllegalArgumentException e) {
      return;
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testGetDistance() {
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 2.0, 1.0);
    double expectedDistance;
    
    expectedDistance = Math.max(Math.max(triangle.getPointA()[Y],
                                         triangle.getPointB()[Y]),
                                triangle.getPointC()[Y]);
    
    assertEquals(expectedDistance, triangle.getDistance(), PRECISION);
  }
  
  public void testGetAngleOf() {
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 1.0, 1.0);
    double expectedAngleAHoriz = -90.0;
    double expectedAngleBHoriz = 0.0;
    double expectedAngleCHoriz = 0.0;
    double expectedAngleAVert = 0.0;
    double expectedAngleBVert = 0.0;
    double expectedAngleCVert = 45.0;
    Double angle;
    
    Class c = Triangle.class;
    Class[] params = {double[].class, int.class};
    Object[] args1 = {triangle.getPointA(), new Integer(HORIZONTAL)};
    Object[] args2 = {triangle.getPointB(), new Integer(HORIZONTAL)};
    Object[] args3 = {triangle.getPointC(), new Integer(HORIZONTAL)};
    Object[] args4 = {triangle.getPointA(), new Integer(VERTICAL)};
    Object[] args5 = {triangle.getPointB(), new Integer(VERTICAL)};
    Object[] args6 = {triangle.getPointC(), new Integer(VERTICAL)};
    
    try {
      Method method = c.getDeclaredMethod("getAngleOf", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      angle = (Double)method.invoke(triangle, args1);
      assertEquals(expectedAngleAHoriz, angle.doubleValue(), PRECISION);
      
      angle = (Double)method.invoke(triangle, args2);
      assertEquals(expectedAngleBHoriz, angle.doubleValue(), PRECISION);
      
      angle = (Double)method.invoke(triangle, args3);
      assertEquals(expectedAngleCHoriz, angle.doubleValue(), PRECISION);
      
      angle = (Double)method.invoke(triangle, args4);
      assertEquals(expectedAngleAVert, angle.doubleValue(), PRECISION);
      
      angle = (Double)method.invoke(triangle, args5);
      assertEquals(expectedAngleBVert, angle.doubleValue(), PRECISION);
      
      angle = (Double)method.invoke(triangle, args6);
      assertEquals(expectedAngleCVert, angle.doubleValue(), PRECISION);
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testGetAngleOfInvalid() {
    Triangle triangle = new Triangle(1.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0,
                                     0.0, 0.0, 1.0);
    
    Class c = Triangle.class;
    Class[] params = {double[].class, int.class};
    Object[] args = {triangle.getPointA(), new Integer(15)};
    
    try {
      Method method = c.getDeclaredMethod("getAngleOf", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      method.invoke(triangle, args);
    } catch (InvocationTargetException ite) {
      if (ite.getTargetException().toString().endsWith("IllegalArgumentException")) //$NON-NLS-1$
        return;
    } catch (Exception e) {
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
    fail("Failed: expected IllegalArgumentException."); //$NON-NLS-1$
  }
  
  public void testSetColor() {
    Color expectedColor = new Color(255, 0, 0);
    Triangle triangle = new Triangle(1.0, 1.0, 3.0,
                                     1.0, 3.0, 1.0,
                                     3.0, 1.0, 1.0);
    
    triangle.setColor(expectedColor);
    
    assertEquals(expectedColor, triangle.getColor());
  }

  public void testSetPointA() {
    double[] expected = {4.0, 4.0, 4.0};
    Triangle triangle = new Triangle(1.0, 1.0, 3.0,
                                     1.0, 3.0, 1.0,
                                     3.0, 1.0, 1.0);

    triangle.setPointA(expected[X], expected[Y], expected[Z]);
    
    assertEquals(expected[X], triangle.getPointA()[X], PRECISION);
    assertEquals(expected[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(expected[Z], triangle.getPointA()[Z], PRECISION);
  }

  public void testSetPointB() {
    double[] expected = {4.0, 4.0, 4.0};
    Triangle triangle = new Triangle(1.0, 1.0, 3.0,
                                     1.0, 3.0, 1.0,
                                     3.0, 1.0, 1.0);

    triangle.setPointB(expected[X], expected[Y], expected[Z]);
    
    assertEquals(expected[X], triangle.getPointB()[X], PRECISION);
    assertEquals(expected[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(expected[Z], triangle.getPointB()[Z], PRECISION);
  }

  public void testSetPointC() {
    double[] expected = {4.0, 4.0, 4.0};
    Triangle triangle = new Triangle(1.0, 1.0, 3.0,
                                     1.0, 3.0, 1.0,
                                     3.0, 1.0, 1.0);

    triangle.setPointC(expected[X], expected[Y], expected[Z]);
    
    assertEquals(expected[X], triangle.getPointC()[X], PRECISION);
    assertEquals(expected[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(expected[Z], triangle.getPointC()[Z], PRECISION);
  }

  public void testTurn() {
    double[] p1 = {1.0, 1.0, 3.0};
    double[] p2 = {1.0, 3.0, 1.0};
    double[] p3 = {3.0, 1.0, 1.0};
    double[] ref1 = {p1[X], p1[Y], p1[Z]};
    double[] ref2 = {p2[X], p2[Y], p2[Z]};
    double[] ref3 = {p3[X], p3[Y], p3[Z]};
    double angle = 15.0;
    double rad = Math.toRadians(angle);
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    
    triangle.turn(angle, LEFTRIGHT);
    
    // pointA
    assertEquals(ref1[X] * Math.cos(rad) + ref1[Y] * Math.sin(rad), triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y] * Math.cos(rad) - ref1[Y] * Math.sin(rad), triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z], triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X] * Math.cos(rad) + ref2[Y] * Math.sin(rad), triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y] * Math.cos(rad) - ref2[X] * Math.sin(rad), triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z], triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X] * Math.cos(rad) + ref3[Y] * Math.sin(rad), triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y] * Math.cos(rad) - ref3[X] * Math.sin(rad), triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z], triangle.getPointC()[Z], PRECISION);
    
    triangle.turn(-angle, LEFTRIGHT);
    
    // pointA
    assertEquals(ref1[X], triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z], triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X], triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z], triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X], triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z], triangle.getPointC()[Z], PRECISION);
    
    triangle.turn(angle, UPDOWN);
    
    // pointA
    assertEquals(ref1[X], triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y] * Math.cos(rad) + ref1[Z] * Math.sin(rad), triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z] * Math.cos(rad) - ref1[Y] * Math.sin(rad), triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X], triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y] * Math.cos(rad) + ref2[Z] * Math.sin(rad), triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z] * Math.cos(rad) - ref2[Y] * Math.sin(rad), triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X], triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y] * Math.cos(rad) + ref3[Z] * Math.sin(rad), triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z] * Math.cos(rad) - ref3[Z] * Math.sin(rad), triangle.getPointC()[Z], PRECISION);
    
    triangle.turn(-angle, UPDOWN);
    
    // pointA
    assertEquals(ref1[X], triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z], triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X], triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z], triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X], triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testMove() {
    double[] p1 = {1.0, 1.0, 3.0};
    double[] p2 = {1.0, 3.0, 1.0};
    double[] p3 = {3.0, 1.0, 1.0};
    double[] ref1 = {p1[X], p1[Y], p1[Z]};
    double[] ref2 = {p2[X], p2[Y], p2[Z]};
    double[] ref3 = {p3[X], p3[Y], p3[Z]};
    double step = 1.0;
    Triangle triangle = new Triangle(p1[X], p1[Y], p1[Z],
                                     p2[X], p2[Y], p2[Z],
                                     p3[X], p3[Y], p3[Z]);
    
    triangle.move(step, X);
    
    // pointA
    assertEquals(ref1[X] + step, triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z], triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X] + step, triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z], triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X] + step, triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z], triangle.getPointC()[Z], PRECISION);
    
    triangle.move(step, Y);
    
    // pointA
    assertEquals(ref1[X] + step, triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y] + step, triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z], triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X] + step, triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y] + step, triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z], triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X] + step, triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y] + step, triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z], triangle.getPointC()[Z], PRECISION);
    
    triangle.move(step, Z);
    
    // pointA
    assertEquals(ref1[X] + step, triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y] + step, triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z] + step, triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X] + step, triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y] + step, triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z] + step, triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X] + step, triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y] + step, triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z] + step, triangle.getPointC()[Z], PRECISION);
    
    triangle.move(-step, Y);
    
    // pointA
    assertEquals(ref1[X] + step, triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z] + step, triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X] + step, triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z] + step, triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X] + step, triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z] + step, triangle.getPointC()[Z], PRECISION);
    
    triangle.move(-step, X);
    
    // pointA
    assertEquals(ref1[X], triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z] + step, triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X], triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z] + step, triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X], triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z] + step, triangle.getPointC()[Z], PRECISION);
    
    triangle.move(-step, Z);
    
    // pointA
    assertEquals(ref1[X], triangle.getPointA()[X], PRECISION);
    assertEquals(ref1[Y], triangle.getPointA()[Y], PRECISION);
    assertEquals(ref1[Z], triangle.getPointA()[Z], PRECISION);
    // pointB
    assertEquals(ref2[X], triangle.getPointB()[X], PRECISION);
    assertEquals(ref2[Y], triangle.getPointB()[Y], PRECISION);
    assertEquals(ref2[Z], triangle.getPointB()[Z], PRECISION);
    // pointC
    assertEquals(ref3[X], triangle.getPointC()[X], PRECISION);
    assertEquals(ref3[Y], triangle.getPointC()[Y], PRECISION);
    assertEquals(ref3[Z], triangle.getPointC()[Z], PRECISION);
  }
  
  public void testIsEqual() {
    double[] p1 = {1.0, 2.0, 3.0};
    double[] p2 = {1.0, 2.0, 3.0};
    double[] p3 = {2.0, 2.0, 3.0};
    double[] p4 = {1.0, 3.0, 3.0};
    double[] p5 = {1.0, 2.0, 4.0};
    Triangle triangle = new Triangle(p3[X], p3[Y], p3[Z],
                                     p4[X], p4[Y], p4[Z],
                                     p5[X], p5[Y], p5[Z]);
    Boolean result;
    
    Class c = triangle.getClass();
    Class[] params = {double[].class, double[].class};
    Object[] args1 = {p1, p2};
    Object[] args2 = {p1, p3};
    Object[] args3 = {p1, p4};
    Object[] args4 = {p1, p5};
    
    try{
      Method method = c.getDeclaredMethod("isEqual", params); //$NON-NLS-1$
      method.setAccessible(true);
      
      result = (Boolean)method.invoke(triangle, args1);
      assertTrue(result.booleanValue());
      
      result = (Boolean)method.invoke(triangle, args2);
      assertFalse(result.booleanValue());
      
      result = (Boolean)method.invoke(triangle, args3);
      assertFalse(result.booleanValue());
      
      result = (Boolean)method.invoke(triangle, args4);
      assertFalse(result.booleanValue());
    } catch(Exception e){
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }

  public void testSin() {
    double angle1 = -10.0;
    double angle2 = 0.0;
    double angle3 = 35.0;
    double angle4 = 182.0;
    double angle5 = 360.0;
    double angle6 = 361.2;
    Triangle triangle = new Triangle(1.0, 1.0, 3.0,
                                     1.0, 3.0, 1.0,
                                     3.0, 1.0, 1.0);
    Double result;
    
    Class c = triangle.getClass();
    Class[] params = {double.class};
    Object[] args1 = {new Double(angle1)};
    Object[] args2 = {new Double(angle2)};
    Object[] args3 = {new Double(angle3)};
    Object[] args4 = {new Double(angle4)};
    Object[] args5 = {new Double(angle5)};
    Object[] args6 = {new Double(angle6)};
    
    try{
      Method sine = c.getDeclaredMethod("sin", params); //$NON-NLS-1$
      sine.setAccessible(true);
      
      result = (Double)sine.invoke(triangle, args1);
      assertEquals(Math.sin(Math.toRadians(angle1)), result.doubleValue(), PRECISION);
      
      result = (Double)sine.invoke(triangle, args2);
      assertEquals(Math.sin(Math.toRadians(angle2)), result.doubleValue(), PRECISION);
      
      result = (Double)sine.invoke(triangle, args3);
      assertEquals(Math.sin(Math.toRadians(angle3)), result.doubleValue(), PRECISION);
      
      result = (Double)sine.invoke(triangle, args4);
      assertEquals(Math.sin(Math.toRadians(angle4)), result.doubleValue(), PRECISION);
      
      result = (Double)sine.invoke(triangle, args5);
      assertEquals(Math.sin(Math.toRadians(angle5)), result.doubleValue(), PRECISION);
      
      result = (Double)sine.invoke(triangle, args6);
      assertEquals(Math.sin(Math.toRadians(angle6)), result.doubleValue(), PRECISION);
    } catch(Exception e){
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }
  
  public void testCos() {
    double angle1 = -50.0;
    double angle2 = 0.0;
    double angle3 = 50.0;
    double angle4 = 235.1;
    double angle5 = 360.0;
    double angle6 = 360.1;
    Triangle triangle = new Triangle(1.0, 1.0, 3.0,
                                     1.0, 3.0, 1.0,
                                     3.0, 1.0, 1.0);
    Double result;
    
    Class c = triangle.getClass();
    Class[] params = {double.class};
    Object[] args1 = {new Double(angle1)};
    Object[] args2 = {new Double(angle2)};
    Object[] args3 = {new Double(angle3)};
    Object[] args4 = {new Double(angle4)};
    Object[] args5 = {new Double(angle5)};
    Object[] args6 = {new Double(angle6)};
    
    try{
      Method cosine = c.getDeclaredMethod("cos", params); //$NON-NLS-1$
      cosine.setAccessible(true);
      
      result = (Double)cosine.invoke(triangle, args1);
      assertEquals(Math.cos(Math.toRadians(angle1)), result.doubleValue(), PRECISION);
      
      result = (Double)cosine.invoke(triangle, args2);
      assertEquals(Math.cos(Math.toRadians(angle2)), result.doubleValue(), PRECISION);
      
      result = (Double)cosine.invoke(triangle, args3);
      assertEquals(Math.cos(Math.toRadians(angle3)), result.doubleValue(), PRECISION);
      
      result = (Double)cosine.invoke(triangle, args4);
      assertEquals(Math.cos(Math.toRadians(angle4)), result.doubleValue(), PRECISION);
      
      result = (Double)cosine.invoke(triangle, args5);
      assertEquals(Math.cos(Math.toRadians(angle5)), result.doubleValue(), PRECISION);
      
      result = (Double)cosine.invoke(triangle, args6);
      assertEquals(Math.cos(Math.toRadians(angle6)), result.doubleValue(), PRECISION);
    } catch(Exception e){
      fail("Failed: method inaccessible."); //$NON-NLS-1$
    }
  }

}
