/*
 * OverviewPanel.java
 */
 
package de.ba_ravensburg.gdv.view;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;

import javax.swing.JPanel;

import de.ba_ravensburg.gdv.model.Scene;

/**
 * @author Max Schloss
 */
public class OverviewPanel extends JPanel {

  private final float PANEL_EDGE = 210f;
  private final float REAL_EDGE  = 275f;
  private final float TRANSFORM_FAKT = (PANEL_EDGE / REAL_EDGE);
  private int x, y, aperture;
  private double[] direction;
  
  private Graphics bufferGraphics;
  private Image bufferImage;
  
  public OverviewPanel() {
    super();

    Dimension dim = new Dimension((int)PANEL_EDGE, (int)PANEL_EDGE);
    this.setPreferredSize(dim);
    this.setMinimumSize(dim);
    this.setMaximumSize(dim);
    this.setDoubleBuffered(false);
  }
  
  private void resetBuffer() {
    // clean up the previous image
    if (bufferGraphics != null) {
      bufferGraphics.dispose();
      bufferGraphics = null;
    }
    if (bufferImage != null) {
      bufferImage.flush();
      bufferImage = null;
    }

    // create the new image with the size of the panel
    bufferImage = createImage((int)PANEL_EDGE, (int) PANEL_EDGE);
    bufferGraphics = bufferImage.getGraphics();
  }
  
  public void update(Graphics g) {
     paint(g);
  }
  
  public void paint(double[] thePos, double theAperture,
                    double[] theDirection) {
    x = (int)Math.round(thePos[0]);
    y = (int)Math.round(thePos[1]);
    aperture = (int)Math.round(theAperture);
    direction = theDirection;
    
    this.paintImmediately(0, 0, (int)PANEL_EDGE, (int)PANEL_EDGE);
  }
  
  public void paintComponent(Graphics g) {
    int camPosX, camPosY;

    if (bufferImage == null || bufferGraphics == null)
      resetBuffer();

    bufferGraphics.setColor(Scene.GROUND_COLOR);
    bufferGraphics.fillRect(0, 0, (int)PANEL_EDGE, (int)PANEL_EDGE);
     
    // draw cube
    bufferGraphics.setColor(new Color(255, 51, 0));
    bufferGraphics.fillRect(Math.round((40 + 50) * TRANSFORM_FAKT),
                            Math.round(PANEL_EDGE - (40 + 50) * TRANSFORM_FAKT -
                                       20 * TRANSFORM_FAKT),
                            Math.round(20 * TRANSFORM_FAKT),
                            Math.round(20 * TRANSFORM_FAKT));
    bufferGraphics.setColor(Color.DARK_GRAY);
    bufferGraphics.drawRect(Math.round((40 + 50) * TRANSFORM_FAKT),
                            Math.round(PANEL_EDGE - (40 + 50) * TRANSFORM_FAKT -
                                       20 * TRANSFORM_FAKT),
                            Math.round(20 * TRANSFORM_FAKT),
                            Math.round(20 * TRANSFORM_FAKT));
    
    // draw column
    bufferGraphics.setColor(new Color(255, 204, 51));
    bufferGraphics.fillPolygon(new int[] {Math.round((90 + 50) *
                                                     TRANSFORM_FAKT),
                                          Math.round((110 + 50) *
                                                     TRANSFORM_FAKT),
                                          Math.round((120 + 50) *
                                                     TRANSFORM_FAKT)},
                               new int[] {Math.round(PANEL_EDGE - (130 + 50) *
                                                     TRANSFORM_FAKT),
                                          Math.round(PANEL_EDGE - (110 + 50) *
                                                     TRANSFORM_FAKT),
                                          Math.round(PANEL_EDGE - (140 + 50) *
                                                     TRANSFORM_FAKT)},
                               3);
    bufferGraphics.setColor(Color.DARK_GRAY);
    bufferGraphics.drawPolygon(new int[] {Math.round((90 + 50) *
                                                     TRANSFORM_FAKT),
                                          Math.round((110 + 50) *
                                                     TRANSFORM_FAKT),
                                          Math.round((120 + 50) *
                                                     TRANSFORM_FAKT)},
                               new int[] {Math.round(PANEL_EDGE - (130 + 50) *
                                                     TRANSFORM_FAKT),
                                          Math.round(PANEL_EDGE - (110 + 50) *
                                                     TRANSFORM_FAKT),
                                          Math.round(PANEL_EDGE - (140 + 50) *
                                                     TRANSFORM_FAKT)},
                               3);
    
    // draw pyramid
    bufferGraphics.setColor(new Color(0, 102, 204));
    bufferGraphics.fillRect(Math.round((50 + 50) * TRANSFORM_FAKT),
                            Math.round(PANEL_EDGE - (195 + 50) *
                                       TRANSFORM_FAKT),
                            Math.round(30 * TRANSFORM_FAKT),
                            Math.round(20 * TRANSFORM_FAKT));
    bufferGraphics.setColor(Color.DARK_GRAY);
    bufferGraphics.drawRect(Math.round((50 + 50) * TRANSFORM_FAKT),
                            Math.round(PANEL_EDGE - (195 + 50) *
                                       TRANSFORM_FAKT),
                            Math.round(30 * TRANSFORM_FAKT),
                            Math.round(20 * TRANSFORM_FAKT));
    bufferGraphics.drawLine(Math.round((50 + 50) * TRANSFORM_FAKT),
                            Math.round(PANEL_EDGE - (195 + 50) *
                                       TRANSFORM_FAKT),
                            Math.round((80 + 50) * TRANSFORM_FAKT),
                            Math.round(PANEL_EDGE - (175 + 50) *
                                       TRANSFORM_FAKT));
    bufferGraphics.drawLine(Math.round((50 + 50) * TRANSFORM_FAKT),
                            Math.round(PANEL_EDGE - (175 + 50) *
                                       TRANSFORM_FAKT),
                            Math.round((80 + 50) * TRANSFORM_FAKT),
                            Math.round(PANEL_EDGE - (195 + 50) *
                                       TRANSFORM_FAKT));
    
    // Draw camera, with aperture
    int newEdgeX1 = 0;
    int newEdgeY1 = 0;
    int newEdgeX2 = 0;
    int newEdgeY2 = 0;
    
    double vBetrag;
    double theta;
    double aHalbe;
    double rad90;
    double theta1;
    double theta2;
    
    if (direction != null) {
      // Umrechnung in Polarkoordinate
      vBetrag = Math.sqrt(Math.pow(direction[0], 2.0) +
                          Math.pow(direction[1], 2.0)) * 500;
      theta = Math.atan2(direction[0], direction[1]);
      
      aHalbe = (Math.PI * aperture / 180);
      rad90 = Math.PI * (-90) / 180;
      theta1 = rad90 + theta + aHalbe;
      theta2 = rad90 + theta - aHalbe;
      
      // Umrechnung in kartesische Koordinate
      newEdgeX1 = (int) Math.round(vBetrag * Math.cos(theta1));
      newEdgeY1 = (int) Math.round(vBetrag * Math.sin(theta1));
      newEdgeX2 = (int) Math.round(vBetrag * Math.cos(theta2));
      newEdgeY2 = (int) Math.round(vBetrag * Math.sin(theta2));
    }
    
    bufferGraphics.setColor(Color.WHITE);
    camPosX = Math.round((x + 50) * TRANSFORM_FAKT);
    camPosY = Math.round(PANEL_EDGE - ((y + 50) * TRANSFORM_FAKT));
      
    bufferGraphics.drawLine(camPosX, camPosY,
                            camPosX + newEdgeX1, camPosY + newEdgeY1);
    bufferGraphics.drawLine(camPosX, camPosY,
                            camPosX + newEdgeX2, camPosY + newEdgeY2);
      
    Graphics2D g2d = (Graphics2D) bufferGraphics;
    g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
                                                0.25f));
    g2d.fillPolygon(new int[] {camPosX,
                               camPosX + newEdgeX1,
                               camPosX + newEdgeX2},
                    new int[] {camPosY,
                               camPosY + newEdgeY1,
                               camPosY + newEdgeY2},
                    3);
    
    g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f));
    
    g.drawImage(bufferImage, 0, 0, this);
  }

}
