/*
 * View3DPanel.java
 */
 
package de.ba_ravensburg.gdv.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JPanel;

import de.ba_ravensburg.gdv.controller.Triangle2D;

/**
 * @author Max Schloss
 */
public class View3DPanel extends JPanel {

  public static final int SIZE = 542;
  public static final Color BG_COLOR = new Color(51, 51, 51);
  
  private LinkedList triangles2d = null;
  private Graphics bufferGraphics;
  private Image bufferImage;
  private Iterator it;
  private Triangle2D triangle;
  
  public View3DPanel() {
    super();
    Dimension dim = new Dimension(SIZE, SIZE);
    this.setPreferredSize(dim);
    this.setDoubleBuffered(false);
  }
  
  private void resetBuffer() {
    // clean up the previous image
    if (bufferGraphics != null) {
      bufferGraphics.dispose();
      bufferGraphics = null;
    }
    if (bufferImage != null) {
      bufferImage.flush();
      bufferImage = null;
    }

    // create the new image with the size of the panel
    bufferImage = createImage(SIZE, SIZE);
    bufferGraphics = bufferImage.getGraphics();
  }
  
  public void paint(LinkedList theTriangles2d) {
    triangles2d = theTriangles2d;
    this.paintImmediately(0, 0, SIZE, SIZE);
  }
  
  public void update(Graphics g) {
    // overridden for performance
    paint(g);
  }
  
  public void paint(Graphics g) {
    super.paint(g);
    
    if (bufferImage == null || bufferGraphics == null)
      resetBuffer();

    bufferGraphics.setColor(BG_COLOR);
    bufferGraphics.fillRect(0, 0, SIZE, SIZE);
    if (triangles2d != null) {
      it = triangles2d.iterator();
      while (it.hasNext()) {
        triangle = (Triangle2D)it.next();
        bufferGraphics.setColor(triangle.getColor());
        bufferGraphics.fillPolygon(triangle);
      }
    }
    
    g.drawImage(bufferImage, 0, 0, this);
  }

}
